#!/usr/bin/env ruby
#
# LaunchBar Action Script
#
require 'json'
require 'date'

class DateTime
  def today?
    now = Time.now
    today = now.strftime('%Y%m%d')

    return self.strftime('%Y%m%d') == today
  end
end

def doing_show(section, limit=9)
  items = []
  output = %x{/usr/local/bin/doing show #{section} -c #{limit} -s desc -o json}
  data = JSON.parse(output)
  data['items'].each {|entry|
    item = {}
    date = DateTime.parse(entry['date'])
    if date.today?
      formatted = date.strftime('%H:%M')
    else
      formatted = date.strftime('%m/%d')
    end

    icon = "font-awesome:hourglass-start"
    title = entry['title']
    if title =~ /@done/
      icon = "font-awesome:check-square"
      title.sub!(/@done(\(.*?\))?/,'')
      # title.gsub!(/@(\S+)/,'\1')
    end
    item['title'] = %Q{#{formatted}: #{entry['title']}}
    item['subtitle'] = entry['note']
    item['badge'] = entry['time']
    # item['label'] = entry['tags'].map {|t| "@#{t}" }.join(", ")
    item['icon'] = icon
    item['alwaysShowsSubtitle'] = true
    items.push(item)
  }

  timers = []
  data['timers'].each {|timer|
    item = {}
    item['title'] = "@" + timer['tag']
    item['badge'] = timer['formatted']
    item['icon'] = "font-awesome:hourglass-end"
    timers.push(item)
  }

  items.push({ 'title' => 'Tags and Timers', 'children' => timers })

  items
end

def doing_parse(doing)
  items = []

  if doing =~ /^show\b/
    return doing_show(doing.sub(/^show\s*/,''), 0)
  end

  cmd = ['/usr/local/bin/doing --stdout']
  if doing =~ /^done/i
    cmd.push('done')
    cmd.push(doing.sub(/^done/i,''))
  elsif doing =~ /^finish$/i
    cmd.push('finish')
  elsif doing =~ /^meanwhile/i
    cmd.push('meanwhile')
    cmd.push(doing.sub(/^meanwhile/i,''))
  else
    cmd.push('now')
    cmd.push(doing.sub(/^now/i,''))
  end
  result = %x{#{cmd.join(" ")}}
  item = {}
  item['title'] = result
  item['icon'] = "font-awesome:cogs"
  items.push(item)

  items
end

items = []


if ARGV.length == 0
  items = doing_show('currently')
else
  doing = ARGV.join(" ").strip
  items = doing_parse(doing)
end


puts items.to_json
